// Docu : http://wiki.moxiecode.com/index.php/TinyMCE:Create_plugin/3.x#Creating_your_own_plugins

(function() {
	tinymce.create('tinymce.plugins.DFF_FP', {
		/**
		 * Initializes the plugin, this will be executed after the plugin has been created.
		 * This call is done before the editor instance has finished it's initialization so use the onInit event
		 * of the editor instance to intercept that event.
		 *
		 * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		 * @param {string} url Absolute URL to where the plugin is located.
		 */
		init : function(ed, url) {
			// Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mceExample');

			ed.addCommand('mceDFF_FP', function() {
				ed.windowManager.open({
					file : url + '/window.php',
					width : 450,
					height : 300,
					inline : 1
				}, {
					plugin_url : url // Plugin absolute URL
				});
			});

			// Register example button
			ed.addButton('DFF_FP', {
				title : 'DataFeedFile.com Featured Product Shortcode Generator',
				cmd : 'mceDFF_FP',
				image : url + '/dff.blue.png'
			});

			// Add a node change handler, selects the button in the UI when a image is selected
			ed.onNodeChange.add(function(ed, cm, n) {
				cm.setActive('DFF_FP', n.nodeName == 'IMG');
			});
		},

		/**
		 * Returns information about the plugin as a name/value array.
		 * The current keys are longname, author, authorurl, infourl and version.
		 *
		 * @return {Object} Name/value array containing information about the plugin.
		 */
		getInfo : function() {
			return {
					longname  : 'DataFeedFile.com Featured Product Shortcode Generator Plugin',
					author    : 'Dominic Barnes',
					authorurl : 'http://www.datafeedfile.com',
					infourl   : 'http://www.datafeedfile.com',
					version   : "0.5b"
			};
		}
	});

	// Register plugin
	tinymce.PluginManager.add('DFF_FP', tinymce.plugins.DFF_FP);
})();


